################################################################################################
#                                                                                              #
#   BioStand v.3.0                                                                             #
#   Developed on R 4.1.2 (x86-64bit)                                                           #
#   March, 2022                                                                                #
#                                                                                              #
#                                                                                              #
#   Authors:                                                                                   #
#   Walter Zupa, Isabella Bitetto, Maria Teresa Spedicato                                      #
#   Coispa Tecnologia & Ricerca - Stazione sperimentale per lo Studio delle Risorse del Mare   #                        
#                                                                                              #
#   For info and suggestions please contact zupa@coispa.it                                     #
#                                                                                              #
#   doi: 10.5281/zenodo.6392940                                                                #
#                                                                                              #
################################################################################################

rm(list=ls(all=TRUE))

library(sp)
library(raster)
library(rgdal)
library(hms)
library(mgcv)
library(fmsb)
library(Hmisc)
library(tcltk)
library(svDialogs)

#--------------------------------------------------------------
# Selection of the working directory
#--------------------------------------------------------------
wd <- tk_choose.dir(getwd(), "Choose the folder containing the BioIndex routine")
setwd(wd)
source(paste(wd,"/standardization/functions.r", sep=""))
#--------------------------------------------------------------
# Preparation of merge TA-TB file
#--------------------------------------------------------------
merge_TATB <- merge_TATB_function()

#--------------------------------------------------------------
# grid loading
#--------------------------------------------------------------
grid <- load_grid()

#--------------------------------------------------------------
# selection of the response variable
#--------------------------------------------------------------
response_variables <- select_response_variable()
response <- response_variables[[1]]
dependent <- response_variables[[2]]

#--------------------------------------------------------------
# creation of metaDB
#--------------------------------------------------------------
data <- create_metaDB()

#--------------------------------------------------------------
# depth range selection
#--------------------------------------------------------------
source(paste(wd,"/standardization/depth_range.r", sep=""))

#--------------------------------------------------------------
# explorative analysis of data
#--------------------------------------------------------------
source(paste(wd,"/standardization/explorative_analysis.r", sep=""))

#--------------------------------------------------------------
# transformation of response variable
#--------------------------------------------------------------
source(paste(wd,"/standardization/transformation.r", sep=""))

#--------------------------------------------------------------
# choise of the refernce month for standardization
#--------------------------------------------------------------
month_st <- month_resp()

#--------------------------------------------------------------
#--------------------------------------------------------------
# Stepwise selection of models
#--------------------------------------------------------------
#--------------------------------------------------------------
variables <- data.frame(variables = colnames(data)); variables

mod1 <- gam(response ~ s(X)+0, family= gaussian (link = identity),data=data, select=T, gamma = 1.4)
mod2 <- gam(response ~ s(Y)+0, family= gaussian (link = identity),data=data, select=T, gamma = 1.4)
mod3 <- gam(response ~ s(depth)+0, family= gaussian (link = identity),data=data, select=T, gamma = 1.4)
mod4 <- gam(response ~ factor(year)+0, family= gaussian (link = identity),data=data, select=T, gamma = 1.4)
mod5 <- gam(response ~ factor(month)+0, family= gaussian (link = identity),data=data, select=T, gamma = 1.4)
mod6 <- gam(response ~ s(hour)+0, family= gaussian (link = identity),data=data, select=T, gamma = 1.4)

# explained deviance and GCV should be inspected from the summary of the models

summary(mod1)
summary(mod2)
summary(mod3)
summary(mod4)
summary(mod5)
summary(mod6)

AIC(mod1)
AIC(mod2)
AIC(mod3)
AIC(mod4)
AIC(mod5)
AIC(mod6)


# once that the best basic model was detected, the variables should be included one at the time in the same way
#.....
# repeat until the best model was detected

#--------------------------------------------------------------
#--------------------------------------------------------------
tranf_type
family <- "tw"  # "gaussian"  "quasipoisson"  "tw"
linkfun <- "log" # "identity"  "log" "inverse
formula_best <- 'response ~ s(depth, bs = "ps", by = n_haul) + s(X, bs = "cr") + factor(year) + 0'

mod <- mod_estimation(formula_best,family, linkfun)
summary(mod)
#--------------------------------------------------------------
# Inspection of residuals of the final model
#--------------------------------------------------------------
residuals_inspection()

#--------------------------------------------------------------
# plot of splines
#--------------------------------------------------------------
plot_splines()

#--------------------------------------------------------------
# plot of factors
#--------------------------------------------------------------
plot_factors()

#--------------------------------------------------------------
# Prediction of the standardized indices
#--------------------------------------------------------------

source(paste(wd,"/standardization/Index_Prediction.R", sep=""), encoding = 'UTF-8')
