###########   maps directory creation ###########################

setwd(paste(wd,"output_GAM", sep="/"))
dir_maps <- "maps"

if (file.exists(dir_maps)){
  setwd(wd)
} else {
  dir.create(file.path(getwd(), dir_maps), showWarnings = FALSE)
  setwd(wd)
}

#################################################################
###########   raster directory creation ###########################

setwd(paste(wd,"output_GAM", sep="/"))
dir_raster <- "raster"

if (file.exists(dir_raster)){
  setwd(wd)
} else {
  dir.create(file.path(getwd(), dir_raster), showWarnings = FALSE)
  setwd(wd)
}

#################################################################

YEAR <- res_table[i,1]

grid_map <- grid[!is.na(grid$stratum),]
grid_map$year <- res_table[i,1]
grid_map$month <- month_st
grid_map$hour <- 12

par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0)) 

if (transformation =="1" & linkfun=="identity") { 
  grid_map$pred <- predict(mod, newdata = grid_map,se.fit = TRUE)[[1]]
}

# sqrt transformation
if (transformation =="2" & linkfun=="identity") { 
  grid_map$pred <- (predict(mod, newdata = grid_map,se.fit = TRUE)[[1]])^2
}

# log transformation or link
if (transformation =="3") { 
  grid_map$pred <- exp(predict(mod, newdata = grid_map,se.fit = TRUE)[[1]])-1
}
if (linkfun=="log") { 
  grid_map$pred <- exp(predict(mod, newdata = grid_map,se.fit = TRUE)[[1]])
}

# inverse transformation
if (transformation =="4" | linkfun =="inverse") { 
  grid_map$pred <- 1/predict(mod, newdata = grid_map,se.fit = TRUE)[[1]]
}

# quadratic transformation
if (transformation =="5" & linkfun =="identity") { 
  grid_map$pred <- sqrt(predict(mod, newdata = grid_map,se.fit = TRUE)[[1]])
}

# write.table(grid_map, paste(wd, "/output_GAM/raster/",YEAR,"_",sspp,"_(",dependent,")_",depth_range[1],"-",depth_range[2],"_prediction_GRID.csv", sep = ""), sep=";", row.names=FALSE)

view <- grid_map[, c("X","Y","pred")]
coordinates(view) <- ~ X + Y
gridded(view) <- TRUE
raster_ok <- raster(view)

writeRaster(raster_ok, filename=paste(wd, "/output_GAM/raster/",YEAR,"_",sspp,"_(",dependent,")_",depth_range[1],"-",depth_range[2],"_prediction_GRID.tiff", sep = ""), format="GTiff", overwrite=TRUE)

lx =coord_range[1,"xmax"] - coord_range[1,"xmin"]
ly =coord_range[1,"ymax"] - coord_range[1,"ymin"]
ratio <- ly/lx*1.1
img_width <- 30
img_height <- img_width*ratio

if (dependent =="biomass" | dependent =="2"){
  dep_text <- "biomass kg/km^2"
} else { dep_text <- "abundance n/km^2" }

jpeg(file=paste(wd, "/output_GAM/maps/",YEAR,"_",sspp,"_(",dependent,")_",depth_range[1],"-",depth_range[2],"_prediction_GRID.jpg", sep = ""), width=img_width, height=img_height, bg="white", units="cm",res=300)
main_label <- paste(YEAR,"_",sspp,"_GSA",GSA,"_(",dep_text,")_",depth_range[1],"-",depth_range[2], sep="")
plot(raster_ok, main=main_label, xlim=c(coord_range[1,1], coord_range[1,2]), ylim=c(coord_range[1,3], coord_range[1,4]),  xlab=expression(paste("Longitude (",degree,"E)")), ylab=expression(paste("Latitude (",degree,"N)")))
plot(continent, col="grey", add=T)
plot(d3, col="#00ffff", add=T)
plot(d4, col="#1e90ff", add=T)
plot(d5, col="blue", add=T)
box()
dev.off()

###############

df_bubble <- data[data$year == YEAR , c("X","Y","response") ]

jpeg(file=paste(wd, "/output_GAM/maps/",YEAR,"_",sspp,"_(",dependent,")_",depth_range[1],"-",depth_range[2],"_observations.jpg", sep = ""), width=img_width, height=img_height, bg="white", units="cm",res=300)
source(paste(wd,"/standardization/bubbleplot.R", sep=""), encoding = 'UTF-8')
dev.off()