#----------------
# functions
#----------------

load_grid <- function(){
        n_grid <- menu(c("0.03 degrees", "0.0625 degrees (Copernicus grid)"), title="Choose the grid to be used in the analysis") 
        
        if (n_grid == 1){
            grid <- read.table(paste(wd,"/standardization/utilities/GRID/GRID_(COUNTRY).csv", sep=""), sep=";",header = T)
        } else {
            grid <- read.table(paste(wd,"/standardization/utilities/GRID/GRID_Copernicus_(COUNTRY).csv", sep=""), sep=";",header = T)
        }
        GSA <- unique(merge_TATB$GSA)
        grid$COUNTRY <- as.character(grid$COUNTRY)
        country_analysis <- unique(as.character(merge_TATB$COUNTRY))
        grid <- grid[grid$GSA == GSA & grid$COUNTRY %in% country_analysis, ]
        cat(paste("Grid correctly loaded for GSA", GSA, sep=""))
        return(grid)
}

merge_TATB_function <- function () {
  # select TA file
  file_TA <- dir(path = paste(wd,"input/TA",sep="/"), pattern = "\\.csv$", all.files = FALSE, full.names = FALSE, include.dirs = FALSE, no.. = TRUE)
  TA <- read.table(paste(wd,"input/TA",file_TA, sep="/"), sep=";", header=T)
  cat("TA file correctly read\n")
  
  file_TB <- dir(path = paste(wd,"input/TB",sep="/"), pattern = "\\.csv$", all.files = FALSE, full.names = FALSE, include.dirs = FALSE, no.. = TRUE)
  TB <- read.table(paste(wd,"input/TB",file_TB, sep="/"), sep=";", header=T)
  cat("TB file correctly read\n")
  
  
  #########################
  ###   filtro COUNTRY  ###
  #########################
  TA_country <- sort(unique(as.character(TA$COUNTRY)))
  TB_country <- sort(unique(as.character(TB$COUNTRY)))
  l_country <- length(TA_country)
  
  if (l_country==1){
    
    check_country ="Y"
    country_analysis <<- TA_country
    
  } else {
    
    cat("\f")
    print(data.frame(Countries=TA_country))
    
    check_country <-  menu(c("Yes", "No"), title=  paste("There are ",l_country," countries in the TA file.\n Do you want to perform the analysis on the entire GSA area?",sep="")  )
    # check_country <- toupper(as.character(readline(paste("there are ",l_country," countries in the TA file.\n Do you want to perform the analysis on the entire GSA area? (Y/y) --> "))))
    if (check_country == 1) {
      check_country ="Y"
      country_analysis <<- TA_country
    } else {
      cat("\f")
      check_country ="N"
      # print(data.frame(Countries = TA_country))
      sel_country <- menu(TA_country ,title=  "Select the country for the analysis (use only numbers) -> "  )
      # sel_country <- as.numeric(readline("Select the country for the analysis -> "))
      country_analysis <<- TA_country[sel_country]
    }
  } # close -->if  length(l_country)==1
  
  TA <- TA[TA$COUNTRY %in% country_analysis , ]
  TB <- TB[TB$COUNTRY %in% country_analysis , ]
  #########################
  
  
  source(paste(wd,"standardization/tables_cols.R", sep="/"))
  
  TA_cols <- c(TA_cols,"VESSEL")
  id_TA <- data.frame(id = paste(TA$AREA,TA$COUNTRY,TA$YEAR,"_", TA$VESSEL, TA$MONTH, TA$DAY,"_", TA$HAUL_NUMBER, sep = ""))
  id_TB <- data.frame(id = paste(TB$AREA,TB$COUNTRY,TB$YEAR,"_", TB$VESSEL, TB$MONTH, TB$DAY,"_", TB$HAUL_NUMBER, sep = ""))
  
  colnames(TA)[which(colnames(TA) == "AREA")] <- "GSA"
  colnames(TB)[which(colnames(TB) == "AREA")] <- "GSA"
  
  TA_merge <- cbind(id_TA,TA)
  TB_merge <- cbind(id_TB,TB)
  TB_merge$GENUS <- as.character(TB_merge$GENUS)
  TB_merge$SPECIES <- as.character(TB_merge$SPECIES)
  
  species_response <- toupper(dlgInput("Select the species: ", default="ARISFOL",Sys.info()[""])$res)
  species <- substr(species_response,1,4)
  species[2] <- substr(species_response,5,7)
  
  TA_merge <- TA_merge[,which(colnames(TA_merge) %in% TA_cols)]
  TB_merge <- TB_merge[TB_merge$GENUS == species[1] & TB_merge$SPECIES == species[2], which(colnames(TB_merge) %in% TB_cols)]
  
  ########################
  ##    MERGE TA- TB    ##
  ########################
  cat("Merging TA-TB files\n")
  
  merge_TATB <- merge(TA_merge, TB_merge, by.x = "id", by.y = "id", all.x = T)
  merge_TATB$MEDITS_CODE <- as.character(paste(merge_TATB$GENUS, merge_TATB$SPECIES))
  
  l_TATB <- length (merge_TATB[,1])
  i=1
  for (i in 1:l_TATB){
    if (merge_TATB[i, "MEDITS_CODE"] == "NA NA"){
      merge_TATB[i, "MEDITS_CODE"              ] <- "NA"
      merge_TATB[i, "GENUS"                    ] <- -1
      merge_TATB[i, "SPECIES"                  ] <- -1
      merge_TATB[i, "TOTAL_WEIGHT_IN_THE_HAUL" ] <- 0
      merge_TATB[i, "TOTAL_NUMBER_IN_THE_HAUL" ] <- 0
      merge_TATB[i, "NB_OF_FEMALES"            ] <- 0
      merge_TATB[i, "NB_OF_MALES"              ] <- 0
      merge_TATB[i, "NB_OF_UNDETERMINED"       ] <- 0
    } 
  }
  
  Data <- merge_TATB
  source(paste(wd,"standardization/utilities/COORD_Medits - to - WGS84.R", sep="/"))
  coord <- convert_coordinates(merge_TATB)
  
  merge_TATB$MEAN_LATITUDE <- (merge_TATB$SHOOTING_LATITUDE+merge_TATB$HAULING_LATITUDE)/2
  merge_TATB$MEAN_LATITUDE_DEC <- (coord$lat_start+coord$lat_end)/2
  merge_TATB$MEAN_LONGITUDE <- (merge_TATB$SHOOTING_LONGITUDE+merge_TATB$HAULING_LONGITUDE)/2
  merge_TATB$MEAN_LONGITUDE_DEC <- (coord$lon_start+coord$lon_end)/2
  merge_TATB$MEAN_DEPTH <- (merge_TATB$SHOOTING_DEPTH+merge_TATB$HAULING_DEPTH)/2
  merge_TATB$SWEPT_AREA <- merge_TATB$DISTANCE * merge_TATB$WING_OPENING/10000000
  i=1
  hour_shooting <- 0
  min_shooting <- 0
  hour_hauling <- 0
  min_hauling <- 0
  
  strata_scheme <- read.table(paste(wd, "/standardization/utilities/strata.csv", sep=""), sep=";",header = T)
  strata_scheme <- strata_scheme[strata_scheme$GSA == unique(merge_TATB$GSA) & strata_scheme$COUNTRY %in% as.character(unique(country_analysis))[1], ]
  
  for (i in 1:l_TATB){
    # if  (merge_TATB$MEAN_DEPTH[i] > 10 & merge_TATB$MEAN_DEPTH[i] <= 50) {merge_TATB$STRATUM_CODE[i] <- 1}
    # if  (merge_TATB$MEAN_DEPTH[i] > 50 & merge_TATB$MEAN_DEPTH[i] <= 100) {merge_TATB$STRATUM_CODE[i] <- 2}
    # if  (merge_TATB$MEAN_DEPTH[i] > 100 & merge_TATB$MEAN_DEPTH[i] <= 200) {merge_TATB$STRATUM_CODE[i] <- 3}
    # if  (merge_TATB$MEAN_DEPTH[i] > 200 & merge_TATB$MEAN_DEPTH[i] <= 500) {merge_TATB$STRATUM_CODE[i] <- 4}
    # if  (merge_TATB$MEAN_DEPTH[i] > 500 & merge_TATB$MEAN_DEPTH[i] <= 800) {merge_TATB$STRATUM_CODE[i] <- 5}
    
    for (j in 1:length(strata_scheme$CODE)){
      if  (merge_TATB$MEAN_DEPTH[i] > strata_scheme[j,4] & merge_TATB$MEAN_DEPTH[i] <= strata_scheme[j,5]) {merge_TATB$STRATUM_CODE[i] <- strata_scheme[j,3]}
    }
    
    if (nchar(merge_TATB$SHOOTING_TIME[i])==4) {hour_shooting[i] <- substr(merge_TATB$SHOOTING_TIME[i],1,2); min_shooting[i] <- substr(merge_TATB$SHOOTING_TIME[i],3,4)} else {
      if (nchar(merge_TATB$SHOOTING_TIME[i])==3) {hour_shooting[i] <- substr(merge_TATB$SHOOTING_TIME[i],1,1); min_shooting[i] <- substr(merge_TATB$SHOOTING_TIME[i],2,3)}}
    
    if (nchar(merge_TATB$HAULING_TIME[i])==4) {hour_hauling[i] <- substr(merge_TATB$HAULING_TIME[i],1,2); min_hauling[i] <- substr(merge_TATB$HAULING_TIME[i],3,4)} else {
      if (nchar(merge_TATB$HAULING_TIME[i])==3) {hour_hauling[i] <- substr(merge_TATB$HAULING_TIME[i],1,1); min_hauling[i] <- substr(merge_TATB$HAULING_TIME[i],2,3)}}
  }
  
  hms_shooting <- hms( rep (0, length(hour_shooting)), as.numeric(min_shooting), as.numeric(hour_shooting))
  hms_hauling  <- hms( rep (0, length(hour_shooting)), as.numeric(min_hauling), as.numeric(hour_hauling))
  duration <-  as.numeric(hms_hauling - hms_shooting)/3600
  
  merge_TATB$SHOOTING_TIME <- hms_shooting
  merge_TATB$HAULING_TIME  <- hms_hauling
  
  merge_TATB$N_h <- merge_TATB$TOTAL_NUMBER_IN_THE_HAUL/duration
  merge_TATB$N_km2 <- merge_TATB$TOTAL_NUMBER_IN_THE_HAUL/merge_TATB$SWEPT_AREA
  merge_TATB$kg_h <- merge_TATB$TOTAL_WEIGHT_IN_THE_HAUL/duration/1000
  merge_TATB$kg_km2 <- merge_TATB$TOTAL_WEIGHT_IN_THE_HAUL/merge_TATB$SWEPT_AREA/1000
  
  merge_TATB$MEDITS_CODE <- as.character(merge_TATB$MEDITS_CODE)
  
  write.table(merge_TATB, paste(wd, "/output_GAM/mergeTATB_",species[1], species[2],"_standardization.csv", sep=""), sep=";", row.names=F)
  
  cat("TA-TB files correctly merged\n")
  cat(paste("Merge TA-TB files saved in the following folder: '",wd, "/output_GAM/mergeTATB_",species[1], species[2],"_standardization.csv'","\n", sep=""))
  
  return(merge_TATB)
}

select_response_variable <- function() {
  print("Select the response variable")
  res_response <- dlgInput("Select the response variable: \n(1) n_km2\n(2) kg_km2", default="kg_km2",Sys.info()[""])$res
  response <- NA
  if (res_response == "n_km2" | res_response =="1") {
    response = "N_km2"
    dependent <- "abundance"
  } else {
    if (res_response == "kg_km2" | res_response =="2"){
      response = "kg_km2"
      dependent <- "biomass"
      }
  }
  return(list(response, dependent))
}

vif_func<-function(in_frame,thresh=10,trace=T,...){
  
  library(fmsb)
  
  if(any(!'data.frame' %in% class(in_frame))) in_frame<-data.frame(in_frame)
  
  #get initial vif value for all comparisons of variables
  vif_init<-NULL
  var_names <- names(in_frame)
  for(val in var_names){
    regressors <- var_names[-which(var_names == val)]
    form <- paste(regressors, collapse = '+')
    form_in <- formula(paste(val, '~', form))
    vif_init<-rbind(vif_init, c(val, VIF(lm(form_in, data = in_frame, ...))))
  }
  vif_max<-max(as.numeric(vif_init[,2]), na.rm = TRUE)
  
  if(vif_max < thresh){
    if(trace==T){ #print output of each iteration
      prmatrix(vif_init,collab=c('var','vif'),rowlab=rep('',nrow(vif_init)),quote=F)
      cat('\n')
      cat(paste('All variables have VIF < ', thresh,', max VIF ',round(vif_max,2), sep=''),'\n\n')
    }
    return(var_names)
  }
  else{
    
    in_dat<-in_frame
    
    #backwards selection of explanatory variables, stops when all VIF values are below 'thresh'
    while(vif_max >= thresh){
      
      vif_vals<-NULL
      var_names <- names(in_dat)
      
      for(val in var_names){
        regressors <- var_names[-which(var_names == val)]
        form <- paste(regressors, collapse = '+')
        form_in <- formula(paste(val, '~', form))
        vif_add<-VIF(lm(form_in, data = in_dat, ...))
        vif_vals<-rbind(vif_vals,c(val,vif_add))
      }
      max_row<-which(vif_vals[,2] == max(as.numeric(vif_vals[,2]), na.rm = TRUE))[1]
      
      vif_max<-as.numeric(vif_vals[max_row,2])
      
      if(vif_max<thresh) break
      
      if(trace==T){ #print output of each iteration
        prmatrix(vif_vals,collab=c('var','vif'),rowlab=rep('',nrow(vif_vals)),quote=F)
        cat('\n')
        cat('removed: ',vif_vals[max_row,1],vif_max,'\n\n')
        flush.console()
      }
      
      in_dat<-in_dat[,!names(in_dat) %in% vif_vals[max_row,1]]
      
    }
    
    return(names(in_dat))
    
  }
  
}

create_metaDB <- function() {
  cols_merge_TATB <- c(c("id","COUNTRY","GSA","VESSEL","YEAR","MONTH","HAUL_NUMBER","SHOOTING_TIME","HAUL_DURATION", "SWEPT_AREA",
                       "GENUS","SPECIES","MEAN_LONGITUDE_DEC","MEAN_LATITUDE_DEC","MEAN_DEPTH","STRATUM_CODE"), response)
  data <- merge_TATB[, colnames(merge_TATB) %in% cols_merge_TATB]
  colnames(data) <- c("id","COUNTRY","GSA","vessel","year","month","haul","hour","duration","GENUS","SPECIES","Y","X", "depth", "swept_area",
                      "stratum","response")
  # selection of GSA
  GSA <- unique(data$GSA)
  # selection of species
  sspp <- paste(unique(data[data$GENUS != -1, "GENUS" ]), unique(data[data$SPECIES != -1, "SPECIES" ]), sep="")
  data$GENUS <- factor(data$GENUS)
  data$SPECIES <- factor(data$SPECIES)
  # hour extraction
  data$hour<-as.numeric(substr(as.character(data$hour),1,2))

  data$COUNTRY <- as.factor(as.character(data$COUNTRY))
    
    n_haul <- aggregate(data$id, by=list(data$year), length)
    n_haul[,2] <- paste(n_haul[,2]," hauls",sep="") 
    colnames(n_haul) <- c("year", "n_haul")
    data <- merge(data, n_haul, by="year")
    data$n_haul <- as.factor(data$n_haul)
    
    data.frame(colnames(data))
    
    data <- data[, c(2:5,1,6,8,7,18,9,15,10:14,16,17)] # c(2,3,4,5,1,6,8,7,17,9:16)
    print(summary(data))  
    
  # data$year <- as.numeric(data$year)
  return(data)
}

depth_range_fun <- function () {
  strata_scheme <- read.table(paste(wd, "/standardization/utilities/strata.csv", sep=""), sep=";",header = T)
  strata_scheme <- strata_scheme[strata_scheme$GSA == unique(merge_TATB$GSA) & strata_scheme$COUNTRY %in% as.character(unique(merge_TATB$COUNTRY))[1], ]
  
  # selection of depth range
  # depth <- data.frame(matrix(NA,ncol=4, nrow=5))
  # colnames(depth) <- c("strata", "min", "max", "bul")
  # depth$strata <- c(1,2,3,4,5)
  # depth$min <- c(10,50,100,200,500)
  # depth$max <- c(50,100,200,500,800)
  # print("Select the depth range for the analysis")
  # depth_range <- dlgInput("Depth range for the analysis: ", default="200,800",Sys.info()[""])$res
  # depth_range <- data.frame(range = strsplit(depth_range, ",")); depth_range <- as.numeric(as.character(depth_range[,1]))
  # if (depth_range[2] != 800) {depth_range[2] <- depth_range[2]}
  # data <-  data[data$MEAN_DEPTH>depth_range[1] & data$MEAN_DEPTH<=depth_range[2],]
  # strata_range <- seq(which(depth[,"min"]==depth_range[1]),which(depth[,"max"]==depth_range[2]),1)
  # depth[depth$strata %in% strata_range, "bul"] <- T
  # depth[!(depth$strata %in% strata_range), "bul"] <- F
    depth <- data.frame(matrix(NA,ncol=4, nrow=length(strata_scheme$CODE)))
  colnames(depth) <- c("strata", "min", "max", "bul")
  depth$strata <- strata_scheme$CODE # c(1,2,3,4,5)
  depth$min <-strata_scheme$MIN_DEPTH  # c(10,50,100,200,500)
  depth$max <-strata_scheme$MAX_DEPTH  # c(50,100,200,500,800)
  print("Select the depth range for the analysis")
  depth_range <- dlgInput("Depth range for the analysis: ", default="10,100",Sys.info()[""])$res
  depth_range <- data.frame(range = strsplit(depth_range, ",")); depth_range <- as.numeric(as.character(depth_range[,1]))
  if (depth_range[2] != 800) {depth_range[2] <- depth_range[2]}
  data <-  data[data$MEAN_DEPTH>depth_range[1] & data$MEAN_DEPTH<=depth_range[2],]
  strata_range <- seq(which(depth[,"min"]==depth_range[1]),which(depth[,"max"]==depth_range[2]),1)
  depth[depth$strata %in% strata_range, "bul"] <- T
  depth[!(depth$strata %in% strata_range), "bul"] <- F
  # analysis_stratum1 <- depth[1,4]
  # analysis_stratum2 <- depth[2,4]
  # analysis_stratum3 <- depth[3,4]
  # analysis_stratum4 <- depth[4,4]
  # analysis_stratum5 <- depth[5,4]
  analysis_stratum1 <- NA
  analysis_stratum2 <- NA
  analysis_stratum3 <- NA
  analysis_stratum4 <- NA
  analysis_stratum5 <- NA
  analysis_stratum6 <- NA
  
  analysis_stratum1 <- depth[1,4]
  analysis_stratum2 <- depth[2,4]
  analysis_stratum3 <- depth[3,4]
  analysis_stratum4 <- depth[4,4]
  analysis_stratum5 <- depth[5,4]
  analysis_stratum6 <- depth[6,4]
  
  if (is.na(analysis_stratum1)){depth[1,4]<- F; analysis_stratum1 <- F}
  if (is.na(analysis_stratum2)){depth[2,4]<- F; analysis_stratum2 <- F}
  if (is.na(analysis_stratum3)){depth[3,4]<- F; analysis_stratum3 <- F}
  if (is.na(analysis_stratum4)){depth[4,4]<- F; analysis_stratum4 <- F}
  if (is.na(analysis_stratum5)){depth[5,4]<- F; analysis_stratum5 <- F}
  if (is.na(analysis_stratum6)){depth[6,4]<- F; analysis_stratum6 <- F}
  
  return(list(c(analysis_stratum1,analysis_stratum2, analysis_stratum3,analysis_stratum4,analysis_stratum5,analysis_stratum6), depth_range))
}

residuals_inspection <- function() {
  #-----------------------------
  # inspection of residuals
  #-----------------------------
  
  res <- residuals(mod)
  par(mfrow=c(1,3))
  qq.gam(mod)
  hist(res, xlab = "residuals", main = "Hstogram of residuals")
  plot(res, ylim=c(), ylab = "residuals")
  abline(0,0, col="red", lwd =2)
  
  # saving plot
  main <- paste(sspp,"_GSA",GSA,"_(",dependent,")_",family,"_",tranf_type,"_",linkfun,"__",depth_res[[2]][1],"-",depth_res[[2]][2],"_Residuals.jpg", sep="")
  jpeg(paste(dir_output,main,sep=""), res = 300, width = 12, height = 5, units = 'in', pointsize = 1/300)
  par(mfrow=c(1,1),oma=c(1,1,1,1), mgp=c(2, 1,0)) 
  res <- residuals(mod)
  par(mfrow=c(1,3))
  qq.gam(mod)
  hist(res, xlab = "residuals", main = "Hstogram of residuals")
  plot(res, ylim=c(), ylab = "residuals")
  abline(0,0, col="red", lwd =2)
  dev.off()
  
}

plot_splines <- function() {
  par(mfrow=c(2,3), mar=c(5,5,2,2), mgp=c(3,1, 0))
  plot(mod,scheme=1, scale=0,cex.main=1.4, cex.axis=1.4,cex.lab=1.4, cex=1.7, lwd=1.7,theta=45, phi=40,too.far=1.3)
  
  main <- paste(sspp,"_GSA",GSA,"_(",dependent,")_",family,"_",tranf_type,"_",linkfun,"_","_",depth_res[[2]][1],"-",depth_res[[2]][2], sep="")
  tiff(paste(wd,"/output_GAM/",main,"_splines.jpg",sep=""), res = 300, width = 10, height = 7, units = 'in', compression = 'lzw', pointsize = 1/300)
  par(mfrow=c(2,3), mar=c(5,5,2,2), mgp=c(3,1, 0))
  plot(mod,scheme=1, scale=0,cex.main=1.4, cex.axis=1.4,cex.lab=1.4, cex=1.7, lwd=1.7,theta=45, phi=40,too.far=1.3)
  dev.off()
  
}

plot_factors <- function() {
  par(mfrow=c(2,3), mar=c(5,5,2,2), mgp=c(3,1, 0))
  terms_names <- row.names(summary(mod)$ pTerms.table)
  if (!is.null(terms_names)){
  termplot(mod, se=T, terms= terms_names,cex.main=1.4, cex.axis=1.4,cex.lab=1.4)
  
  main <- paste(sspp,"_GSA",GSA,"_(",dependent,")_",family,"_",tranf_type,"_",linkfun,"_","_",depth_res[[2]][1],"-",depth_res[[2]][2], sep="")
  tiff(paste(wd,"/output_GAM/",main,"_factors.jpg",sep=""), res = 300, width = 10, height = 7, units = 'in', compression = 'lzw', pointsize = 1/300)
  par(mfrow=c(2,3), mar=c(5,5,2,2), mgp=c(3,1, 0))
  termplot(mod, se=T, terms= terms_names,cex.main=1.4, cex.axis=1.4,cex.lab=1.4)
  dev.off()} else{
    warning("No factor variables in the model")
  }
  
}

month_resp <- function() {
  month_st <- as.numeric(dlgInput("Insert the reference month for standardization: ", default="7",Sys.info()[""])$res)
  month <- month_st
  return(month_st)
}

mod_estimation <- function(formula_best,family, linkfun) {
  if (linkfun == "identity") { # link identity
    # family gaussian
    if (family == "gaussian"){
      mod <- gam (formula( formula_best ), data=data, family=gaussian(identity) , select=T, gamma=1.4)
    }  
    if (family =="tw"){
      mod <- gam (formula( formula_best ), data=data, family=tw(), select=T, gamma=1.4, method="REML")
    }
    # family quasipoisson
    if (family == "quasipoisson"){
      mod <- gam (formula( formula_best ), data=data, family=quasipoisson() , select=T, gamma=1.4)
    }
  } # chiusura link identity
  
  
  if (linkfun == "log") { # link log
    if (family == "gaussian"){
      mod <- gam (formula( formula_best ), data=data, family=gaussian(log) , select=T, gamma=1.4)
    }  
    # family tweedie
    if (family =="tw"){
      mod <- gam (formula( formula_best ), data=data, family=tw(), select=T, gamma=1.4, method="REML")
    }
    # family quasipoisson
    if (family == "quasipoisson"){
      mod <- gam (formula( formula_best ), data=data, family=quasipoisson() , select=T, gamma=1.4)
    }
    
  } # chiusura link log
  
  
  if (linkfun == "inverse") { #link inverse
    if (family == "gaussian"){
      mod <- gam (formula( formula_best ), data=data, family=gaussian(inverse) , select=T, gamma=1.4)
    }  
    
  } # chiusura link inverse
  return(mod)
} # function  mod_estimation()






{
# pred_identity <- function() {
#   i=1
#   for(i in 1:length(res_table[,1])){
#     
#     
#     if (analysis_stratum1 == T){
#       stratum_1$year <- res_table[i,1]
#       stratum_1$month <- month_st
#       stratum_1$hour <- 12
#       stratum_1$pred <- predict(mod, newdata = data.frame(stratum_1),se.fit = TRUE)[[1]]
#       res_table[i,2] <- mean(stratum_1$pred) *area_s1
#       se_strata1 <- std(stratum_1$pred) 
#       se_table[i,2] <- se_strata1 *area_s1 } else {
#         area_s1 = 0
#       }
#     
#     if (analysis_stratum2 == T){
#       stratum_2$year <- res_table[i,1]
#       stratum_2$month <- month_st
#       stratum_2$hour <- 12
#       stratum_2$pred <- predict(mod, newdata = data.frame(stratum_2),se.fit = TRUE)[[1]]
#       res_table[i,3] <- mean(stratum_2$pred) *area_s2
#       se_strata2 <- std(stratum_2$pred) 
#       se_table[i,3] <- se_strata2 *area_s2} else {
#         area_s2 = 0
#       }
#     
#     if (analysis_stratum3 == T){
#       stratum_3$year <- res_table[i,1]
#       stratum_3$month <- month_st
#       stratum_3$hour <- 12
#       stratum_3$pred <- predict(mod, newdata = data.frame(stratum_3),se.fit = TRUE)[[1]]
#       res_table[i,4] <- mean(stratum_3$pred) *area_s3
#       se_strata3 <- std(stratum_1$pred) 
#       se_table[i,4] <- se_strata3 *area_s3} else {
#         area_s3 = 0
#       }
#     
#     if (analysis_stratum4 == T){
#       stratum_4$year <- res_table[i,1]
#       stratum_4$month <- month_st
#       stratum_4$hour <- 12
#       stratum_4$pred <- predict(mod, newdata = data.frame(stratum_4),se.fit = TRUE)[[1]]
#       res_table[i,5] <- mean(stratum_4$pred) *area_s4
#       se_strata4 <- std(stratum_4$pred) 
#       se_table[i,5] <- se_strata4 *area_s4} else {
#         area_s4 = 0
#       }
#     
#     if (analysis_stratum5 == T){
#       stratum_5$year <- res_table[i,1]
#       stratum_5$month <- month_st
#       stratum_5$hour <- 12
#       stratum_5$pred <- predict(mod, newdata = data.frame(stratum_5),se.fit = TRUE)[[1]]
#       res_table[i,6] <- mean(stratum_5$pred) *area_s5
#       se_strata5 <- std(stratum_5$pred) 
#       se_table[i,6] <- se_strata5 *area_s5} else {
#         area_s5 = 0
#       }
#     
#     sum_res <- c(res_table[i,2],res_table[i,3],res_table[i,4],res_table[i,5],res_table[i,6])
#     res_table[i, 7]<- sum(sum_res[!is.na(sum_res)])/sum(area_s1,area_s2,area_s3,area_s4,area_s5)
# 
#     sum_se <- c(se_table[i,2],se_table[i,3],se_table[i,4],se_table[i,5],se_table[i,6])
#     se_table[i, 7]<- sum(sum_se[!is.na(sum_se)])/sum(area_s1,area_s2,area_s3,area_s4,area_s5)
#     
#     
#   }
#   return(list(res_table,se_table))
# }
# 
# pred_sqrt <- function() {
#   i=1
#   for(i in 1:length(res_table[,1])){
#     
#     
#     if (analysis_stratum1 == T){
#       stratum_1$year <- res_table[i,1]
#       stratum_1$month <- month_st
#       stratum_1$hour <- 12
#       stratum_1$pred <- (predict(mod, newdata = data.frame(stratum_1),se.fit = TRUE)[[1]])^2
#       res_table[i,2] <- mean(stratum_1$pred) *area_s1
#       se_strata1 <- std(stratum_1$pred) 
#       se_table[i,2] <- se_strata1 *area_s1 } else {
#         area_s1 = 0
#       }
#     
#     if (analysis_stratum2 == T){
#       stratum_2$year <- res_table[i,1]
#       stratum_2$month <- month_st
#       stratum_2$hour <- 12
#       stratum_2$pred <- (predict(mod, newdata = data.frame(stratum_2),se.fit = TRUE)[[1]])^2
#       res_table[i,3] <- mean(stratum_2$pred) *area_s2
#       se_strata2 <- std(stratum_2$pred) 
#       se_table[i,3] <- se_strata2 *area_s2} else {
#         area_s2 = 0
#       }
#     
#     if (analysis_stratum3 == T){
#       stratum_3$year <- res_table[i,1]
#       stratum_3$month <- month_st
#       stratum_3$hour <- 12
#       stratum_3$pred <- (predict(mod, newdata = data.frame(stratum_3),se.fit = TRUE)[[1]])^2
#       res_table[i,4] <- mean(stratum_3$pred) *area_s3
#       se_strata3 <- std(stratum_1$pred) 
#       se_table[i,4] <- se_strata3 *area_s3} else {
#         area_s3 = 0
#       }
#     
#     if (analysis_stratum4 == T){
#       stratum_4$year <- res_table[i,1]
#       stratum_4$month <- month_st
#       stratum_4$hour <- 12
#       stratum_4$pred <- (predict(mod, newdata = data.frame(stratum_4),se.fit = TRUE)[[1]])^2
#       res_table[i,5] <- mean(stratum_4$pred) *area_s4
#       se_strata4 <- std(stratum_4$pred) 
#       se_table[i,5] <- se_strata4 *area_s4} else {
#         area_s4 = 0
#       }
#     
#     if (analysis_stratum5 == T){
#       stratum_5$year <- res_table[i,1]
#       stratum_5$month <- month_st
#       stratum_5$hour <- 12
#       stratum_5$pred <- (predict(mod, newdata = data.frame(stratum_5),se.fit = TRUE)[[1]])^2
#       res_table[i,6] <- mean(stratum_5$pred) *area_s5
#       se_strata5 <- std(stratum_5$pred) 
#       se_table[i,6] <- se_strata5 *area_s5} else {
#         area_s5 = 0
#       }
#     
#     sum_res <- c(res_table[i,2],res_table[i,3],res_table[i,4],res_table[i,5],res_table[i,6])
#     res_table[i, 7]<- sum(sum_res[!is.na(sum_res)])/sum(area_s1,area_s2,area_s3,area_s4,area_s5)
#     
#     sum_se <- c(se_table[i,2],se_table[i,3],se_table[i,4],se_table[i,5],se_table[i,6])
#     se_table[i, 7]<- sum(sum_se[!is.na(sum_se)])/sum(area_s1,area_s2,area_s3,area_s4,area_s5)
#     
#     
#   }
#   return(list(res_table,se_table))
# }
# 
# pred_log<- function() {
#   i=1
#   for(i in 1:length(res_table[,1])){
#     
#     if (analysis_stratum1 == T){
#       stratum_1$year <- res_table[i,1]
#       stratum_1$month <- month_st
#       stratum_1$hour <- 12
#       stratum_1$pred <- exp(predict(mod, newdata = data.frame(stratum_1),se.fit = TRUE)[[1]])
#       res_table[i,2] <- mean(stratum_1$pred) *area_s1
#       se_strata1 <- std(stratum_1$pred) 
#       se_table[i,2] <- se_strata1 *area_s1 } else {
#         area_s1 = 0
#       }
#     
#     if (analysis_stratum2 == T){
#       stratum_2$year <- res_table[i,1]
#       stratum_2$month <- month_st
#       stratum_2$hour <- 12
#       stratum_2$pred <- exp(predict(mod, newdata = data.frame(stratum_2),se.fit = TRUE)[[1]])
#       res_table[i,3] <- mean(stratum_2$pred) *area_s2
#       se_strata2 <- std(stratum_2$pred) 
#       se_table[i,3] <- se_strata2 *area_s2} else {
#         area_s2 = 0
#       }
#     
#     if (analysis_stratum3 == T){
#       stratum_3$year <- res_table[i,1]
#       stratum_3$month <- month_st
#       stratum_3$hour <- 12
#       stratum_3$pred <- exp(predict(mod, newdata = data.frame(stratum_3),se.fit = TRUE)[[1]])
#       res_table[i,4] <- mean(stratum_3$pred) *area_s3
#       se_strata3 <- std(stratum_1$pred) 
#       se_table[i,4] <- se_strata3 *area_s3} else {
#         area_s3 = 0
#       }
#     
#     if (analysis_stratum4 == T){
#       stratum_4$year <- res_table[i,1]
#       stratum_4$month <- month_st
#       stratum_4$hour <- 12
#       stratum_4$pred <- exp(predict(mod, newdata = data.frame(stratum_4),se.fit = TRUE)[[1]])
#       res_table[i,5] <- mean(stratum_4$pred) *area_s4
#       se_strata4 <- std(stratum_4$pred) 
#       se_table[i,5] <- se_strata4 *area_s4} else {
#         area_s4 = 0
#       }
#     
#     if (analysis_stratum5 == T){
#       stratum_5$year <- res_table[i,1]
#       stratum_5$month <- month_st
#       stratum_5$hour <- 12
#       stratum_5$pred <- exp(predict(mod, newdata = data.frame(stratum_5),se.fit = TRUE)[[1]])
#       res_table[i,6] <- mean(stratum_5$pred) *area_s5
#       se_strata5 <- std(stratum_5$pred) 
#       se_table[i,6] <- se_strata5 *area_s5} else {
#         area_s5 = 0
#       }
#     
#     sum_res <- c(res_table[i,2],res_table[i,3],res_table[i,4],res_table[i,5],res_table[i,6])
#     res_table[i, 7]<- sum(sum_res[!is.na(sum_res)])/sum(area_s1,area_s2,area_s3,area_s4,area_s5)
#     
#     sum_se <- c(se_table[i,2],se_table[i,3],se_table[i,4],se_table[i,5],se_table[i,6])
#     se_table[i, 7]<- sum(sum_se[!is.na(sum_se)])/sum(area_s1,area_s2,area_s3,area_s4,area_s5)
#     
#     
#   }
#   return(list(res_table,se_table))
# }
  }